from __future__ import absolute_import, print_function, unicode_literals

import sys
import logging
from .colors import color
from .formatters import ColorFormatter, FormatterWrapper

class ColorStreamHandler(logging.Handler):

    def __init__(self, use_color=None, level=logging.INFO, show_tb=False):

        logging.Handler.__init__(self, level=level)

        if use_color is None:
            use_color = sys.stdout.isatty()

        self.use_color = use_color

        self.setFormatter(ColorFormatter(show_tb=show_tb))

    def emit(self, record):

        if not self.filter(record):
            return

        fmt = self.format(record)

        if record.levelno == logging.INFO:
            stream = sys.stdout
        else:
            stream = sys.stderr

        if isinstance(fmt, (list, tuple)):
            for item in fmt:
                if isinstance(item, color):
                    if self.use_color:
                        with item(stream) as text:
                            stream.write(text)
                    else:
                        stream.write(item.text)
                else:
                    stream.write(item)
                stream.write(' ')
        else:
            stream.write(fmt)

        stream.write('\n')

        if hasattr(stream, 'flush'):
            stream.flush()

def main():
    logger = logging.getLogger()
    logger.setLevel(logging.DEBUG)
    h = ColorStreamHandler(True, logging.DEBUG)
    logger.addHandler(h)

    logger.debug("DEBUG")
    logger.info("INFO")
    logger.warn("WARN")
    logger.error("ERROR")

    FormatterWrapper.wrap(h, prefix=color('prefix |', [color.WHITE, color.BACKGROUND_COLORS[0]]))

    logger.debug("DEBUG")
    logger.info("INFO")
    logger.warn("WARN")
    logger.error("ERROR")

if __name__ == '__main__':
    main()



import sys
from contextlib import contextmanager
from itertools import chain


class partial_color(object):

    def __init__(self, color_cls, c):
        self.color_cls = color_cls
        self.c = tuple(BaseColor.mkcolor(c))

    def __enter__(self):
        color = self.color_cls
        color.set_colors(color.DEFAULT_STREAM, self.c)

    def __exit__(self, *exc_info):
        color = self.color_cls
        color.set_colors(color.DEFAULT_STREAM, [color.DEFAULT])

    def __call__(self, text):
        color = self.color_cls
        return color(text, self.c)

class color_creator(type):
    def __getattr__(self, attr):
        if hasattr(self, attr.upper()):
            return partial_color(self, getattr(self, attr.upper()))
        else:
            raise AttributeError(attr)

try:
    basestring
except NameError:
    basestring = (str, bytes)

class BaseColor(object):
    __metaclass__ = color_creator

    DEFAULT_STREAM = sys.stdout

    def __init__(self, text, colors=()):
        self.text = text
        self.colors = tuple(self.mkcolor(colors))

    def __repr__(self):
        return 'color(%r, %r)' % (self.text, self.colors)

    def __enter__(self):
        self.set_colors(self.DEFAULT_STREAM, self.colors)

    def __exit__(self, *exc_info):
        self.set_colors(self.DEFAULT_STREAM, [self.DEFAULT])

    @contextmanager
    def __call__(self, stream, colors=()):

        self.set_colors(stream, chain(self.colors, colors))
        try:
            yield self.text
        finally:
            self.set_colors(stream, [self.DEFAULT])

    @classmethod
    def mkcolor(cls, format_spec):
        if isinstance(format_spec, int):
            yield format_spec
            return

        if isinstance(format_spec, basestring):
            format_spec = format_spec.split(',')
        for c in format_spec:
            if isinstance(c, int):
                yield c
            elif isinstance(c, basestring):
                if c.isdigit():
                    yield int(c)
                elif hasattr(cls, c.upper()):
                    yield getattr(cls, c.upper())

import sys
py3 = sys.version_info.major == 3
if py3:
    # Metaclass hack
    BaseColor = color_creator('BaseColor', (BaseColor,), {})



from __future__ import print_function, absolute_import
import logging
from .colors import color
import traceback
from clyent.errors import ClyentError

class FormatterWrapper(object):

    def __init__(self, formatter, prefix='', suffix=''):
        self.prefix = prefix
        self.suffix = suffix
        self.formatter = formatter

    def format(self, record):
        result = self.formatter.format(record)
        if not isinstance(result, list):
            result = [result]

        if self.prefix:
            result.insert(0, self.prefix)
        if self.suffix:
            result.append(self.suffix)
        return result

    @classmethod
    def wrap(cls, handler, prefix='', suffix=''):
        old_formatter = handler.formatter or logging.Formatter()
        new_formatter = cls(old_formatter, prefix, suffix)
        handler.setFormatter(new_formatter)


class ColorFormatter(object):

    COLOR_MAP = {'ERROR': (color.BOLD, color.RED),
                 'WARNING': (color.BOLD, color.YELLO),
                 'DEBUG': (color.BOLD, color.BLUE),
                 }

    def color_map(self, header, level):
        header = '[%s]' % header
        if level in self.COLOR_MAP:
            return color(header, self.COLOR_MAP[level])
        else:
            return header

    def __init__(self, show_tb=False):

        if show_tb is False:
            show_tb = (ClyentError, KeyboardInterrupt)
        elif show_tb is True:
            show_tb = ()

        self.show_tb = show_tb

    def format(self, record):
        if record.levelno == logging.INFO:
            header = None
            message = record.getMessage()
        else:
            if record.exc_info:
                err = record.exc_info[1]
                header = type(err).__name__
                result = [self.color_map(header, record.levelname)]
                if getattr(err, 'message', None):
                    result.append(str(err.message))
                elif err.args:
                    result.append(str(err.args[0]))

                if self.show_tb is True or not isinstance(err, self.show_tb):
                    message = ''.join(traceback.format_exception(*record.exc_info))
                    result.append('\n' + message)
                return result

            else:
                header = record.levelname.lower()
                message = record.getMessage()

        if header:
            header = self.color_map(header, record.levelname)
            return [header, '%s' % message]
        else:
            return message



from clyent.logs.colors.base import BaseColor

class PosixColor(BaseColor):
    WHITE = 97
    YELLO = 93
    BLUE = 94
    GREEN = 92
    RED = 91
    BOLD = 1

    DEFAULT = 0

    UNDERLINE = 4

    BACKGROUND_COLORS = list(range(40, 48)) + [100, 102, 104, 105, 106]

    @classmethod
    def set_colors(cls, stream, colors):
        col = list(colors)
        for c in col:
            stream.write('\033[%im' % c)




from clyent.logs.colors.base import BaseColor

try:
    from pywintypes import error as Win32Error
    import win32console
except ImportError:
    std_output_hdl = None
else:
    # std_output_hdl will be none if this is a service process
    std_output_hdl = win32console.GetStdHandle(win32console.STD_OUTPUT_HANDLE)


if std_output_hdl is not None:
    try:
        std_output_hdl.GetConsoleScreenBufferInfo()
    except Win32Error:  # Not a valid ConsoleScreenBuffer
        std_output_hdl = None

class NTColor(BaseColor):
    YELLO = 14
    BLUE = 11
    GREEN = 10
    RED = 12
    BOLD = 0
    WHITE = 15

    DEFAULT = 15

    BACKGROUND_COLORS = [c << 4 for c in range(1, 10)]

    @classmethod
    def set_colors(cls, stream, colors):
        if std_output_hdl is not None:
            c = reduce(lambda a, b: a | b, colors)
            std_output_hdl.SetConsoleTextAttribute(c)

